/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    Identifier.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Map;
import java.util.UUID;

import org.trustedcomputinggroup._2010.ifmap._2.AccessRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.DeviceType;
import org.trustedcomputinggroup._2010.ifmap._2.IPAddressType;
import org.trustedcomputinggroup._2010.ifmap._2.IdentityType;
import org.trustedcomputinggroup._2010.ifmap._2.MACAddressType;

/**
 * Represents an Identifier
 *
 */
public class Identifier {
    
	private ArrayList<LinkContainer> links = new ArrayList<LinkContainer>();
	private String name;
	private Object requestObject;
	private final Hashtable<String, String> attr = new Hashtable<String, String>();
	
	private String hash = null;
	
	private int type;
	
	public Identifier(Object o){		
		if (o instanceof AccessRequestType) {
			name = "access-request";
			AccessRequestType art = (AccessRequestType)o;
			requestObject = art;
			attr.put("name", art.getName());
			this.type = IdentifierConstants.ACCESS_REQUEST;
    	} else if (o instanceof DeviceType) {
    		name = "device";
    		DeviceType d = (DeviceType)o;
    		requestObject = d;
    		attr.put("name", d.getName());
    		if(d.getAikName() != null)
    			attr.put("aik-name", d.getAikName()); 
    		this.type = IdentifierConstants.DEVICE;
    	} else if (o instanceof IPAddressType) {
    		name = "ip-address";
    		IPAddressType ipt = (IPAddressType)o;    		
    		requestObject = ipt;
    		attr.put("value", ipt.getValue());
    		attr.put("type", ipt.getType());  
    		this.type = IdentifierConstants.IP_ADDRESS;
    	} else if (o instanceof IdentityType) {
    		name = "identity";
    		IdentityType it = (IdentityType)o;    
    		requestObject = it;
    		attr.put("name", it.getName());
    		attr.put("type", it.getType());    
    		this.type = IdentifierConstants.IDENTITY;
    	} else if (o instanceof MACAddressType) {
    		name = "mac-address";
    		MACAddressType mt = (MACAddressType)o;
    		requestObject = mt;
    		attr.put("value", mt.getValue());
    		this.type = IdentifierConstants.MAC_ADDRESS;
    	}
    	else {
    		//System.err.println("Identifier: No valid requestObject!");
    	}		
		hash = UUID.randomUUID().toString();
		//
	}
	
	public String getHash(){
		return hash;
	}
	
	public int getType(){
		return type;
	}
	
	public Object getRequestObject() {
		return requestObject;
	}

	public boolean isTheSameLike(Identifier other){
		// Anderer Typ
		if(type != other.getType()) return false;
		
		switch(type){
		case IdentifierConstants.ACCESS_REQUEST:
			if(this.getValue("name").equals(other.getValue("name")))
				return true;
			break;
		case IdentifierConstants.IP_ADDRESS:
			if(this.getValue("value").equals(other.getValue("value"))
					&&
			   this.getValue("type").equals(other.getValue("type"))
			  ) return true;
			break;
		case IdentifierConstants.DEVICE:
			if(this.getValue("name").equals(other.getValue("name")))
				return true;
			break;
		case IdentifierConstants.IDENTITY:
			if(this.getValue("name").equals(other.getValue("name"))
					&&
			   this.getValue("type").equals(other.getValue("type"))
			  ) return true;
			break;
		case IdentifierConstants.MAC_ADDRESS:
			if(this.getValue("value").equals(other.getValue("value")))
				return true;
			break;			
		}
		return false;		
	}
	
	public String getUniqueIdentifierName(){
		return getName()+"\r\n"+getAttributes().toString();
	}
	
	public String toString(){
		StringBuffer buf = new StringBuffer();		
		buf.append(">>> "+name+" <<<");
		buf.append(" [ ");		
		Enumeration<String> iter = attr.keys();
		
		while(iter.hasMoreElements()){
			String key = iter.nextElement();
			String value = attr.get(key);
			buf.append(key + "="+value + " ");
		}
		buf.append("] \r\n");
		if(!this.links.isEmpty()){
			buf.append("Anzahl LinkContainer: "+this.links.size()+"\r\n");		
			for(LinkContainer l : links){
				buf.append("Link: "+l.getUniqueLinkName()+"\r\n");				
			}
			buf.append("\r\n");
		}
		return buf.toString();
	}
	
	public void addLink(LinkContainer l){
		if(l != null && !links.contains(l)){
			//System.err.println("Link added: "+l.getUniqueLinkName());
			links.add(l);
		}
	}
	
	public void addLinks(ArrayList<LinkContainer> conti){
		if(conti != null){
			for(LinkContainer lc : conti){
				if(conti.size() > 0){
					if(!links.contains(lc)){
						links.add(lc);
					}					
				}
			}		
		}
	}
	
	public ArrayList<LinkContainer> getLinks(){
		return links;
	}
	
	public void removeLink(Link l){
		links.remove(l);
	}
	
	public String getValue(String key){
		return attr.get(key);
	}
	
	public Map<String, String> getAttributes(){
		//We return an unmodifiable Map
		return Collections.unmodifiableMap(attr);		
	}
	
	public Collection<String> getValues(){
		return attr.values();
	}
	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
}
